/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#include "config.h"
#include "configfile.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>
#include <cerrno>
#include <cstring>
#include <fstream>
#include <iostream>

void setupSignalHandlers()
{
  struct sigaction act;
  sigset_t set;

  sigemptyset(&set);

  act.sa_handler = SIG_IGN;
  act.sa_mask = set;
  act.sa_flags = 0;

  sigaction(SIGPIPE, &act, 0);
}

bool daemonize(ConfigFile &config)
{
  std::string logfile = config.get("log");
  int fd = open(logfile.c_str(), O_WRONLY|O_APPEND|O_CREAT, 0744);
  if (fd == -1) {
    std::cerr << "Failed to open log file " << logfile << "\n";
    return false;
  }

  close(0);
  close(1);
  close(2);

  int devnull = open("/dev/null", O_RDONLY);

  dup2(devnull, 0);
  dup2(fd, 1);
  dup2(fd, 2);
  close(fd);

  // Fork #1
  pid_t pid = fork();
  if (pid < 0) {
    std::cerr << "Failed to fork: " << strerror(errno) << "\n";
    return false;
  }

  if (pid > 0) 			// Exit the first parent
    exit(0);

  chdir("/");
  umask(022);
  setsid();

  // Fork #2
  pid_t pid2 = fork();
  if (pid2 < 0) {
    std::cerr << "Failed to fork (2): " << strerror(errno) << "\n";
    return false;
  }

  if (pid2 > 0)			// Exit the second parent
    exit(0);
  
  // Write PID file

  std::string pidfile = config.get("pid");
  std::ofstream pidf(pidfile.c_str());
  if (!pidf) 
    std::cerr << "Warning: could not open " << pidfile << " for writing pid\n";
  else {
    pidf << getpid() << "\n";
    pidf.close();
  }
    
  return true;
}
